/*
 * @(#)LadderControl.java	1.00 14th March 1999
 * 
 * Modification Log:
 *
 */


import java.awt.*;
import java.awt.event.*;


/**
 * A simple control to display data in a horizontal ladder shaped format.
 * It has a rounded rectangle border and a bar to separate each data item.
 * A data item consists of a heading and the value. The text displays are
 * centered horizontally.
 * 
 * You are free to use this code and to make modifications provided
 * this notice is retained.
 * <p>
 * If you found this useful, please add a note of acknowledgement to my guestbook,
 * or send me a mail at <a href="mailto:tanmaykm@hotmail.com">tanmaykm@hotmail.com</a>. 
 * If you would like to report a bug or suggest some improvements, 
 * you are most welcome. I will be happy to help you use this piece of code.
 * <p>
 *
 * @author 	    Tanmay K. Mohapatra
 * @version     1.00, 14th March, 1999
 */

public class LadderControl extends java.awt.Component implements java.awt.event.ComponentListener
{
    public  Color        colorBorder;        // color to draw the rounded rectangle border
    public  Color        colorBackgr;        // the background color
    public  Color        colorSeparator;     // color with which to draw the separator bars
    public  Color        colorHeading;       // color in which to print the headings
    public  Color        colorValue;         // color in which to print the values
    public  int          iSeparatorSpacing = 20; // spacing on top & bottom of separator bar
    public  int          iTextSpacing      = 10; // spacing on top & bottom of text items
    public  int          iBorderSpacing    = 5;  // spacing on either side of the border rect

    private Font         fontHeading;        // font to use for the headings
    private Font         fontData;           // font to use for the data

    private boolean     bRecalcReqd = true;      
    private String[]    aHeadings;          // array containing heading strings
    private String[]    aData;              // array containing data strings
    private int[]       aHeadingsPos;       // array containing pre calculated heading y-positions
    private int[]       aDataPos;           // array containing pre calculated data y-positions
    private int[]       aSeparatorPos;      // array containing pre calculated separator y-positions
    private FontMetrics fmHeading, fmData;  // stored font metrics for repeated calculations
    private int         iPreferredHeight, iPreferredWidth;  // pre calculated preferred sizes
    private int         iHeadingAscent, iDataAscent, iHeadingDescent, iDataDescent; // pre calculated font dimensions

    /**
     * initialized the data arrays
     * 
     * @param iNumEntries   number of entries in this instance
     */
    public LadderControl(int iNumEntries)
    {
        aHeadings = new String[iNumEntries];
        aData = new String[iNumEntries];
        aHeadingsPos = new int[iNumEntries];
        aDataPos = new int[iNumEntries];
        aSeparatorPos = new int[iNumEntries-1];
        setFontsAndColors();
        addComponentListener(this);
        
        for (int iIndex=0; iIndex<iNumEntries; iIndex++)
        {
            setData("", "", iIndex);
        }
    }

    /**
     * set the heading and data at the specified position
     */
    public void setData(String sHeading, String sData, int iPos)
    {
        if ((iPos < 0) || (iPos > aHeadings.length))
        {
            return;
        }
        aHeadings[iPos] = sHeading;
        aData[iPos] = sData;
        repaint();
    }

    public Dimension getPreferredSize()
    {
        calcPreferredHeight();
        calcPreferredWidth();
        return new Dimension(iPreferredWidth, iPreferredHeight);
    }

    public Dimension getMinimumSize()
    {
        calcPreferredHeight();
        calcPreferredWidth();
        return new Dimension(iPreferredWidth, iPreferredHeight);
    }

    public void paint(Graphics gGraphics)
    {
        int iWidth = getSize().width;
        int iHeight = getSize().height;

        if (bRecalcReqd)
        {
            calcPositions();
        }
        
        gGraphics.setColor(colorBackgr);
        gGraphics.fillRect(0, 0, iWidth, iHeight);
        gGraphics.setColor(colorBorder);
        gGraphics.drawRoundRect(iBorderSpacing, iBorderSpacing, 
                                iWidth-2*iBorderSpacing, iHeight-2*iBorderSpacing, 
                                3*iBorderSpacing, 3*iBorderSpacing);
        for (int iIndex=0; iIndex < aHeadings.length; iIndex++)
        {
            int iStartPos;
            
            gGraphics.setColor(colorHeading);
            gGraphics.setFont(fontHeading);
            iStartPos = (iWidth-fmHeading.stringWidth(aHeadings[iIndex]))/2;
            gGraphics.drawString(aHeadings[iIndex], iStartPos, aHeadingsPos[iIndex]);
            gGraphics.setColor(colorValue);
            gGraphics.setFont(fontData);
            iStartPos = (iWidth-fmData.stringWidth(aData[iIndex]))/2;
            gGraphics.drawString(aData[iIndex], iStartPos, aDataPos[iIndex]);
            gGraphics.setColor(colorSeparator);
            // if this is not the last one, add the separator spacing also
            if (iIndex < aHeadings.length-1)
            {
                gGraphics.drawLine(2*iBorderSpacing, aSeparatorPos[iIndex], iWidth-2*iBorderSpacing, aSeparatorPos[iIndex]);
            }
        }
    }

    private void calcPreferredWidth()
    {
        iPreferredWidth = 0;
        for (int iIndex=0; iIndex < aHeadings.length; iIndex++)
        {
            int iWidth;
            iWidth = fmHeading.stringWidth(aHeadings[iIndex]);
            if (iPreferredWidth < iWidth)
            {
                iPreferredWidth = iWidth;
            }
            iWidth = fmData.stringWidth(aData[iIndex]);
            if (iPreferredWidth < iWidth)
            {
                iPreferredWidth = iWidth;
            }
        }
        iPreferredWidth += 4*iBorderSpacing;
    }

    private void calcPreferredHeight()
    {
        iPreferredHeight = aHeadings.length*(iHeadingAscent+iHeadingDescent) +
                           aData.length*(iDataAscent+iDataDescent) +
                           (aData.length + aHeadings.length)*2*iTextSpacing +
                           aSeparatorPos.length*2*iSeparatorSpacing;
        iPreferredHeight += 4*iBorderSpacing;
    }

    /**
     * calculate and store positions of different elements so that paint is faster
     */
    private void calcPositions()
    {
        iHeadingAscent = fmHeading.getAscent();
        iDataAscent = fmData.getAscent();
        iHeadingDescent = fmHeading.getDescent();
        iDataDescent = fmData.getDescent();

        calcPreferredHeight();
        int iHeightTillNow=0;
        
        // calculate excess space
        int iExcess = getSize().height - iPreferredHeight;
        
        if (iExcess != 0)
        {
            // distribute it equally
            iExcess /= (aHeadingsPos.length + aDataPos.length + aSeparatorPos.length);
            if (0 == iExcess)
            {
                // excess was too small to be distributed equally. 
                // so center vertically
                iHeightTillNow += (getSize().height - iPreferredHeight)/2;
            }
        }
        
        
        iHeightTillNow += 2*iBorderSpacing;
        for (int iIndex=0; iIndex<aHeadings.length; iIndex++)
        {
            iHeightTillNow += iTextSpacing;
            iHeightTillNow += (iHeadingAscent + iExcess);
            aHeadingsPos[iIndex] = iHeightTillNow;
            iHeightTillNow += iHeadingDescent;
            iHeightTillNow += 2*iTextSpacing;
            iHeightTillNow += (iDataAscent + iExcess);
            aDataPos[iIndex] = iHeightTillNow;
            iHeightTillNow += iDataDescent;
            iHeightTillNow += iTextSpacing;
            
            // if this is not the last one, add the separator spacing also
            if (iIndex < aHeadings.length-1)
            {
                iHeightTillNow += (iSeparatorSpacing + iExcess);
                aSeparatorPos[iIndex] = iHeightTillNow;
                iHeightTillNow += iSeparatorSpacing;
            }
        }
        bRecalcReqd = false;
    }

    /**
     * set the font with which the heading elements will be shown.
     * determine and store the font metrics for that font
     */
    public void setHeadingFont(Font fontHeading)
    {
        this.fontHeading = fontHeading;
        fmHeading = getFontMetrics(fontHeading);
        bRecalcReqd = true;
        repaint();
    }


    /**
     * set the font with which the data elements will be shown.
     * determine and store the font metrics for that font
     */
    public void setDataFont(Font fontData)
    {
        this.fontData = fontData;
        fmData = getFontMetrics(fontData);
        bRecalcReqd = true;
        repaint();
    }


    /**
     * sets defaults for values not set by the user
     */
    private void setFontsAndColors()
    {
        Color fgCol = getForeground();
        Color bgCol = getBackground();
        if(null == colorBorder) colorBorder = fgCol;
        if(null == colorBackgr) colorBackgr = bgCol;
        if(null == colorSeparator) colorSeparator = fgCol;
        if(null == colorHeading) colorHeading = fgCol;
        if(null == colorValue) colorValue = fgCol;
        
        Font defaultFont = getFont();
        if (null == defaultFont) defaultFont = new Font("Arial", Font.PLAIN, 12);
        
        if (null == fontHeading) setHeadingFont(defaultFont);
        if (null == fontData)    setDataFont(defaultFont);
        setBackground(bgCol);
    }

    /**
     * recalculate the positions
     */
    public void componentResized(ComponentEvent ce)
    {
        bRecalcReqd = true;
        repaint();
    }

    public void componentHidden(ComponentEvent ce)
    {
    }

    public void componentMoved(ComponentEvent ce)
    {
    }

    public void componentShown(ComponentEvent ce)
    {
    }
}
